#!/usr/bin/python3

import skimage.restoration
from skimage import img_as_float
import numpy as np
from PIL import Image
import os
from tqdm import tqdm

imagesFolderPath = 'flat-field'
denoiser = 'wavelet'
npArrayFilePath = f'mean_{imagesFolderPath}_{denoiser}.npy'

denoise = getattr(skimage.restoration, f'denoise_{denoiser}')

mean = None
numberOfImagesInMean = 0

imagesFileNames = os.listdir(imagesFolderPath)

def treatImage(imageFileName):
    global mean, numberOfImagesInMean
    imageFilePath = f'{imagesFolderPath}/{imageFileName}'
    imagePil = Image.open(imageFilePath)
    imageNpArray = img_as_float(np.array(imagePil))
    # As the arguments differ from a denoiser to the other should use a match instead.
    imageDenoisedNpArray = denoise(imageNpArray, channel_axis=-1, convert2ycbcr=True, rescale_sigma=True)
    imageNoiseNpArray = imageNpArray - imageDenoisedNpArray
    if mean is None:
        mean = imageNoiseNpArray
    else:
        mean = ((mean * numberOfImagesInMean) + imageNoiseNpArray) / (numberOfImagesInMean + 1)
    numberOfImagesInMean += 1

for imageFileName in tqdm(imagesFileNames):
    treatImage(imageFileName)

with open(npArrayFilePath, 'wb') as f:
    np.save(f, mean)
